#' Bar plot of AUCs from validating data sets 
#'
#' After coclustering optimization, visualize AUCs generated by optimal parameter settings on validating data sets.

#' @param df.auc The \code{data.frame} of AUCs generated by optimal parameter settings on validating data sets, which is returned by \code{cocluster}.
#' @param auc The column name of AUCs in \code{df.auc}.
#' @param thr The AUC threshold, which will be labeled in the bar plot.
#' @param bar.width Width of a single bar.
#' @param spd.sel A character vector of selected \code{spd.set}, usually the common \code{spd.set} displaying desired AUCs across multiple validating data sets.
#' @param title The title of composite violin plots.
#' @param key.title The title of legend.
#' @param x.agl,x.vjust The angle and vertical position of x-axis text.

#' @return An object of ggplot.

#' @examples

#' # Create parameter settings and corresponding aucs.
#' df.auc <- data.frame(auc=runif(n=5, min=1e-12, max=.99))
#' ran <- seq(0.2, 0.8, 0.1)
#' df.spd <- data.frame(sim=sample(ran, 5, replace=FALSE), sim.p=sample(ran, 5, replace=FALSE), dim=sample(seq(5, 40, 1), 5, replace=FALSE))
#' df.spd$spd.set <- paste0('s', df.spd$sim, 'p', df.spd$sim.p, 'd', df.spd$dim)
#' 
#' df.auc <- cbind(df.spd, df.auc)
#' # Plot the aucs.
#' auc_bar(df.auc, auc='auc', thr=0.5) 

#' # See package vignette by calling "browseVignettes('spatialHeatmap')".

#' @author Jianhai Zhang \email{jzhan067@@ucr.edu} \cr Dr. Thomas Girke \email{thomas.girke@@ucr.edu}

#' @references 
#' H. Wickham. ggplot2: Elegant Graphics for Data Analysis. Springer-Verlag New York, 2016.

#' @importFrom ggplot2 ggplot aes geom_bar geom_text position_dodge2 labs theme element_text geom_hline 

#' @export auc_bar

auc_bar <- function(df.auc, auc='auc', thr=0.5, bar.width=0.8, spd.sel=NULL, title=NULL, key.title=NULL, x.agl=80, x.vjust=0.6) { 
  spd.set <- NULL
  cna <- colnames(df.auc); cna[cna==auc] <- 'auc'
  colnames(df.auc) <- cna
  # df0 <- subset(df.auc, sc==tar.datset)
  df.auc$auc <- round(as.numeric(df.auc$auc), 2)
  df.auc$spd.sel <- 0
  df.auc$spd.sel[df.auc$spd.set %in% spd.sel] <- 1 
  gg <- ggplot(df.auc, aes(x=spd.set, y=auc)) +
  geom_bar(color='black', width=bar.width, fill='#FF6666', position=position_dodge2(width=bar.width, preserve="single"), stat="identity") +
  geom_text(aes(label=ifelse(spd.sel, "*", "")), 
position = position_dodge2(width=bar.width, preserve="single"), vjust=-0.1, size=10) +
  labs(title=title, x="Optimized parameter sets", y='AUC', fill=key.title)+theme(plot.title = element_text(hjust = 0.5)) +
  theme(axis.text.x=element_text(angle=x.agl, vjust=x.vjust)) +
  geom_hline(yintercept=thr, linetype="dashed", color = "black", size=1)
  gg
}
